## vim:ts=4:et:nowrap
##
##---------------------------------------------------------------------------##
##
## PySol -- a Python Solitaire game
##
## Copyright (C) 2000 by T. Kirk
## Copyright (C) 2000 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1999 Markus Franz Xaver Johannes Oberhumer
## Copyright (C) 1998 Markus Franz Xaver Johannes Oberhumer
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; see the file COPYING.
## If not, write to the Free Software Foundation, Inc.,
## 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
##
## Markus F.X.J. Oberhumer
## <markus.oberhumer@jk.uni-linz.ac.at>
## http://wildsau.idv.uni-linz.ac.at/mfx/pysol.html
##
## T. Kirk
## <grania@mailcity.com>
## http://www.inetarena.com/~grania
##
##---------------------------------------------------------------------------##


# Imports
import sys, math, time

# PySol imports
if sys.modules.has_key("pysoltk"):
    from gamedb import registerGame, GameInfo, GI
    from util import *
    from stack import *
    from game import Game
    from layout import Layout
    from hint import AbstractHint, DefaultHint, CautiousDefaultHint
    from pysoltk import MfxCanvasText, getFont


# /***********************************************************************
#  * Dashavatara Foundation Stacks
#  ***********************************************************************/

class Dashavatara_FoundationStack(AbstractFoundationStack):
    def __init__(self, x, y, game, suit, **cap):
        kwdefault(cap, max_move=0, max_cards=12)
        apply(SS_FoundationStack.__init__, (self, x, y, game, suit), cap)

    def updateText(self):
        AbstractFoundationStack.updateText(self)
        self.game.updateText()


class Journey_Foundation(AbstractFoundationStack):
    def __init__(self, x, y, game, suit, **cap):
        kwdefault(cap, mod = 12, dir = 0, base_rank = NO_RANK, max_move = 0)
        apply(AbstractFoundationStack.__init__, (self, x, y, game, suit), cap)

    def acceptsCards(self, from_stack, cards):
        if not AbstractFoundationStack.acceptsCards(self, from_stack, cards):
            return 0
        if not self.cards:
            return 1
        stack_dir = self.game.getFoundationDir()
        if stack_dir == 0:
            card_dir = (cards[0].rank - self.cards[-1].rank) % self.cap.mod
            return card_dir in (1, 11)
        else:
            return (self.cards[-1].rank + stack_dir) % self.cap.mod == cards[0].rank


# /***********************************************************************
#  * Dashavatara Row Stacks
#  ***********************************************************************/

class Dashavatara_OpenStack(OpenStack):
    def __init__(self, x, y, game, **cap):
        kwdefault(cap, max_move = 12, max_accept = 12)
        apply(OpenStack.__init__, (self, x, y, game), cap)
        self.CARD_YOFFSET = self.game.app.images.CARD_YOFFSET

    def canMoveCards(self, cards):
        return (self.basicCanMoveCards(cards)
                and self._isMoveableSequence(cards))

    def currentForce(self, card):
        hour = time.localtime(time.time())[3]
        if hour >= 7 and hour <= 19:
            strong, weak = 0, 1
        else:
            strong, weak = 1, 0
        if card.suit <= 4:
            return strong
        else:
            return weak


class Circles_RowStack(SS_RowStack):
    def __init__(self, x, y, game, base_rank):
        SS_RowStack.__init__(self, x, y, game, base_rank = base_rank,
                                max_accept = 1, max_move = 1)
        self.CARD_YOFFSET = 1


class Journey_BraidStack(OpenStack):
    def __init__(self, x, y, game, xoffset, yoffset):
        OpenStack.__init__(self, x, y, game)
        CW = self.game.app.images.CARDW
        self.CARD_YOFFSET = int(self.game.app.images.CARD_YOFFSET * yoffset)
        # use a sine wave for the x offsets
        self.CARD_XOFFSET = []
        j = 1
        for i in range(30):
            self.CARD_XOFFSET.append(int(math.sin(j) * xoffset))
            j = j + 0.9


class Journey_StrongStack(ReserveStack):
    def fillStack(self):
        if not self.cards:
            if self.game.s.braidstrong.cards:
                self.game.moveMove(1, self.game.s.braidstrong, self)
            elif self.game.s.braidweak.cards:
                self.game.moveMove(1, self.game.s.braidweak, self)

    def getBottomImage(self):
        return self.game.app.images.getBraidBottom()


class Journey_WeakStack(ReserveStack):
    def fillStack(self):
        if not self.cards:
            if self.game.s.braidweak.cards:
                self.game.moveMove(1, self.game.s.braidweak, self)
            elif self.game.s.braidstrong.cards:
                self.game.moveMove(1, self.game.s.braidstrong, self)

    def getBottomImage(self):
        return self.game.app.images.getBraidBottom()


class Journey_ReserveStack(ReserveStack):
    def acceptsCards(self, from_stack, cards):
        if (from_stack is self.game.s.braidstrong or
                from_stack is self.game.s.braidweak or
                from_stack in self.game.s.rows):
            return 0
        return ReserveStack.acceptsCards(self, from_stack, cards)

    def getBottomImage(self):
        return self.game.app.images.getTalonBottom()


class AlternateForce_RowStack(Dashavatara_OpenStack):
    def acceptsCards(self, from_stack, cards):
        if not self.basicAcceptsCards(from_stack, cards):
            return 0
        stackcards = self.cards
        if stackcards:
            return (self.currentForce(stackcards[-1]) != self.currentForce(cards[0])
                        and stackcards[-1].rank - 1 == cards[0].rank)
        else:
            return 1

    def _isMoveableSequence(self, cards):
        for i in range(len(cards) - 1):
            if (self.currentForce(cards[i]) == self.currentForce(cards[i + 1])
                        or cards[i].rank - 1 != cards[i + 1].rank):
                return 0
        return 1


# /***********************************************************************
#  *
#  ***********************************************************************/

class AbstractDashavataraGame(Game):

    SUITS = ("Fish", "Tortoise", "Boar", "Lion", "Dwarf",
             "Axe", "Arrow", "Plough", "Lotus", "Horse")
    RANKS = ("Ace", "2", "3", "4", "5", "6", "7", "8", "9", "10", "Wazir", "Mir")
    COLORS = ("Black", "Red", "Yellow", "Green", "Brown",
              "Orange", "Grey", "White", "Olive", "Crimson")
    FORCE = ("Strong", "Weak")

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.suit == card2.suit
                and (card1.rank + 1 == card2.rank
                or card1.rank - 1 == card2.rank))


class Journey_Hint(DefaultHint):
    # FIXME: demo is not too clever in this game
    pass


# /***********************************************************************
#  * Dashavatara Circles
#  ***********************************************************************/

class DashavataraCircles(AbstractDashavataraGame):

    #
    # Game layout
    #

    def createGame(self):
        l, s = Layout(self), self.s
        font = getFont("canvas_card", cardw=l.CW)

        # Set window size
        self.setSize(l.XM * 4 + l.XS * 8, l.YM + l.YS * 6)

        # Create row stacks
        x = self.width / 2 - l.XS / 2
        y = self.height / 2 - l.YS / 2
        x0 = (-.5, .25, .5, -.25,
            -1.29, -1.1, -.4, .4, 1.1, 1.29, 1.1, .4, -.4, -1.1,
            -2.1, -1.95, -1.5, -.8, 0, .8, 1.5, 1.95, 2.1, 1.95, 1.5, .8, 0, -.8, -1.5, -1.95)
        y0 = (-.3, -.45, .3, .45,
            0, -.8, -1.25, -1.25, -.8, 0, .8, 1.25, 1.25, .8,
            0, -.8, -1.5, -1.95, -2.05, -1.95, -1.5, -.8, 0, .8, 1.5, 1.95, 2.05, 1.95, 1.5, .8)
        for i in range(30):
            s.rows.append(Circles_RowStack(x + l.XS * x0[i] + l.XM * x0[i] * 2,
                            y + l.YS * y0[i] + l.YM * y0[i] * 2,
                            self, base_rank = ANY_RANK))

        # Create reserve stacks
        s.reserves.append(ReserveStack(l.XM, self.height - l.YS, self))
        s.reserves.append(ReserveStack(self.width - l.XS, self.height - l.YS, self))


        # Create foundations
        x, y = l.XM, l.YM
        for j in range(2):
            for i in range(5):
                s.foundations.append(SS_FoundationStack(x, y, self, i + j * 5, mod = 12,
                                             max_move = 0, max_cards = 12))
                y = y + l.YS
            x, y = self.width - l.XS, l.YM
        self.setRegion(s.rows, (l.XM + l.XS, 0, l.XS * 7, 999999))

        # Create talon
        s.talon = InitialDealTalonStack(l.XM + l.XS, l.YM, self)

        # Define stack groups
        l.defaultStackGroups()

    #
    # Game over rides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 120
        for i in range(3):
            self.s.talon.dealRow(frames=0)
        self.startDealSample()
        self.s.talon.dealRow()
        self.s.talon.dealCards()


# /***********************************************************************
#  * Ten Avatars
#  ***********************************************************************/

class TenAvatars(AbstractDashavataraGame):

    #
    # Game layout
    #

    def createGame(self):
        l, s = Layout(self), self.s
        font = getFont("canvas_card", cardw=l.CW)

        # Set window size
        self.setSize(l.XM * 3 + l.XS * 11, l.YM + l.YS * 6)

        # Create row stacks
        x = l.XM
        y = l.YM
        for i in range(10):
            s.rows.append(RK_RowStack(x, y, self, base_rank = 11,
                                        max_move = 12, max_cards = 99))
            x = x + l.XS

        # Create reserve stacks
        x = self.width - l.XS
        y = l.YM
        for i in range(6):
            s.reserves.append(ReserveStack(x, y, self))
            y = y + l.YS
        y = y - l.YS
        for i in range(6):
            x = x - l.XS
            s.reserves.append(ReserveStack(x, y, self))

        self.setRegion(s.rows, (0, 0, l.XM + l.XS * 10, l.YS * 5))

        # Create talon
        s.talon = DealRowTalonStack(l.XM, self.height - l.YS, self)
        l.createText(s.talon, "nn")

        # Define stack groups
        l.defaultStackGroups()

    #
    # Game over rides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 120
        for i in range(4):
            self.s.talon.dealRow(frames=0)
        self.startDealSample()
        self.s.talon.dealCards()

    def isGameWon(self):
        if len(self.s.talon.cards):
            return 0
        for s in self.s.rows:
            if len(s.cards) != 12 or not isSameSuitSequence(s.cards):
                return 0
        return 1


# /***********************************************************************
#  * Balarama
#  ***********************************************************************/

class Balarama(AbstractDashavataraGame):
    Layout_Method = Layout.ghulamLayout
    Talon_Class = InitialDealTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = AC_RowStack

    BASE_RANK = ANY_RANK

    #
    # Game layout
    #

    def createGame(self, **layout):
        l, s = Layout(self), self.s
        kwdefault(layout, rows = 16, reserves = 4, texts = 0)
        apply(self.Layout_Method, (l,), layout)
        self.setSize(l.size[0], l.size[1])

        # Create foundations
        for r in l.s.foundations:
            s.foundations.append(self.Foundation_Class(r.x, r.y, self,
                                    r.suit, mod = 12, max_cards = 12))

        # Create reserve stacks
        for r in l.s.reserves:
            s.reserves.append(ReserveStack(r.x, r.y, self, ))

        # Create row stacks
        for r in l.s.rows:
            s.rows.append(self.RowStack_Class(r.x, r.y, self, suit = ANY_SUIT,
                                base_rank = self.BASE_RANK, max_cards = 12))

        # Create talon
        s.talon = self.Talon_Class(l.s.talon.x, l.s.talon.y, self)

        # Define stack groups
        l.defaultAll()

    #
    # Game over rides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 120
        for i in range(6):
            self.s.talon.dealRow(frames=0)
        self.startDealSample()
        self.s.talon.dealRow()
        self.s.talon.dealRow(rows=self.s.rows[:8])
        self.s.talon.dealCards()

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.color % 2 != card2.color % 2 and
                (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank))


# /***********************************************************************
#  * Hayagriva
#  ***********************************************************************/

class Hayagriva(Balarama):
    Layout_Method = Layout.ghulamLayout
    Talon_Class = InitialDealTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = RK_RowStack

    BASE_RANK = 11

    #
    # Game layout
    #

    def createGame(self, **layout):
        Balarama.createGame(self)

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank)


# /***********************************************************************
#  * Shanka
#  ***********************************************************************/

class Shanka(Balarama):
    Layout_Method = Layout.ghulamLayout
    Talon_Class = InitialDealTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = RK_RowStack

    BASE_RANK = 11

    #
    # Game layout
    #

    def createGame(self, **layout):
        Balarama.createGame(self, reserves = 0)

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank)


# /***********************************************************************
#  * Surukh
#  ***********************************************************************/

class Surukh(Balarama):
    Layout_Method = Layout.ghulamLayout
    Talon_Class = InitialDealTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = AlternateForce_RowStack
    BASE_RANK = ANY_RANK

    #
    # Game layout
    #

    def createGame(self, **layout):
        Balarama.createGame(self, reserves = 4)

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        force1 = card1.suit > 4
        force2 = card2.suit > 4
        return (force1 != force2
                and (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank))


# /***********************************************************************
#  * Matsya
#  ***********************************************************************/

class Matsya(AbstractDashavataraGame):
    Layout_Method = Layout.generiklonLayout
    Talon_Class = WasteTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = RK_RowStack

    BASE_RANK = 11

    #
    # Game layout
    #

    def createGame(self, max_rounds = 1, num_deal = 1, **layout):
        l, s = Layout(self), self.s
        kwdefault(layout, rows = 10, waste = 1)
        apply(self.Layout_Method, (l,), layout)
        self.setSize(l.size[0], l.size[1])

        # Create talon
        s.talon = self.Talon_Class(l.s.talon.x, l.s.talon.y, self,
                            max_rounds = max_rounds, num_deal = num_deal)
        s.waste = WasteStack(l.s.waste.x, l.s.waste.y, self)

        # Create foundations
        for r in l.s.foundations:
            s.foundations.append(self.Foundation_Class(r.x, r.y, self,
                                    r.suit, mod = 12, max_cards = 12, max_move = 0))

        # Create row stacks
        for r in l.s.rows:
            s.rows.append(self.RowStack_Class(r.x, r.y, self,
                                suit = ANY_SUIT, base_rank = self.BASE_RANK))

        # Define stack groups
        l.defaultAll()

    #
    # Game over rides
    #

    def startGame(self):
        assert len(self.s.talon.cards) == 120
        for i in range(10):
            self.s.talon.dealRow(rows=self.s.rows[i+1:], flip=0, frames=0)
        self.startDealSample()
        self.s.talon.dealRow()
        self.s.talon.dealCards()

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank)


# /***********************************************************************
#  * Kurma
#  ***********************************************************************/

class Kurma(Matsya):
    Layout_Method = Layout.generiklonLayout
    Talon_Class = WasteTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = SS_RowStack

    BASE_RANK = ANY_RANK

    #
    # Game layout
    #

    def createGame(self, **layout):
        Matsya.createGame(self, max_rounds = -1)


# /***********************************************************************
#  * Varaha
#  ***********************************************************************/

class Varaha(Matsya):
    Layout_Method = Layout.generiklonLayout
    Talon_Class = WasteTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = SS_RowStack

    BASE_RANK = ANY_RANK

    #
    # Game layout
    #

    def createGame(self, **layout):
        Matsya.createGame(self, max_rounds = -1, num_deal = 3)


# /***********************************************************************
#  * Narasimha
#  ***********************************************************************/

class Narasimha(Matsya):
    Layout_Method = Layout.generiklonLayout
    Talon_Class = WasteTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = AC_RowStack

    BASE_RANK = 11

    #
    # Game layout
    #

    def createGame(self, **layout):
        Matsya.createGame(self, max_rounds = 1, num_deal = 1)

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.color % 2 != card2.color % 2
                and (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank))


# /***********************************************************************
#  * Vamana
#  ***********************************************************************/

class Vamana(Matsya):
    Layout_Method = Layout.generiklonLayout
    Talon_Class = WasteTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = AC_RowStack

    BASE_RANK = 11

    #
    # Game layout
    #

    def createGame(self, **layout):
        Matsya.createGame(self, max_rounds = -1, num_deal = 3)

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.color % 2 != card2.color % 2
                and (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank))


# /***********************************************************************
#  * Parashurama
#  ***********************************************************************/

class Parashurama(Matsya):
    Layout_Method = Layout.generiklonLayout
    Talon_Class = WasteTalonStack
    Foundation_Class = SS_FoundationStack
    RowStack_Class = RK_RowStack

    BASE_RANK = 11

    #
    # Game layout
    #

    def createGame(self, **layout):
        Matsya.createGame(self, max_rounds = 2, num_deal = 3)

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.rank + 1 == card2.rank
                or card2.rank + 1 == card1.rank)


# /***********************************************************************
# // Journey to Cuddapah
# ************************************************************************/

class Journey(AbstractDashavataraGame):
    Hint_Class = Journey_Hint

    BRAID_CARDS = 15
    BRAID_OFFSET = 1.0

    #
    # game layout
    #

    def createGame(self):
        # create layout
        l, s = Layout(self), self.s

        # set window
        # (piles up to 20 cards are playable - needed for Braid_BraidStack)
        decks = self.gameinfo.decks
        h = max(5 * l.YS + 35, l.YS + (self.BRAID_CARDS - 1) * l.YOFFSET)
        self.setSize(l.XM + l.XS * (7 + decks * 2), l.YM + h)

        # extra settings
        self.base_card = None

        # Create foundations, rows, reserves
        s.addattr(braidstrong = None)      # register extra stack variable
        s.addattr(braidweak = None)      # register extra stack variable
        x, y = l.XM, l.YM
        for j in range(5):
            for i in range(decks):
                s.foundations.append(Journey_Foundation(x + l.XS * i, y, self,
                                                j, mod = 12, max_cards = 12))
            s.rows.append(Journey_StrongStack(x + l.XS * decks, y, self))
            s.rows.append(Journey_ReserveStack(x + l.XS * (1 + decks), y, self))
            y = y + l.YS
        x, y = x + l.XS * (5 + decks), l.YM
        for j in range(5):
            s.rows.append(Journey_ReserveStack(x, y, self))
            s.rows.append(Journey_WeakStack(x + l.XS, y, self))
            for i in range(decks, 0, -1):
                s.foundations.append(Journey_Foundation(x + l.XS * (1 + i), y, self,
                                                j + 5, mod = 12, max_cards = 12))
            y = y + l.YS
        self.texts.info = MfxCanvasText(self.canvas,
                               self.width / 2, h - l.YM / 2, anchor = "center",
                               font = getFont("canvas_card", cardw = l.CW))

        # Create braids
        x, y = l.XM + l.XS * 2.15 + l.XS * decks, l.YM
        s.braidstrong = Journey_BraidStack(x, y, self, xoffset = 12, yoffset = self.BRAID_OFFSET)
        x = x + l.XS * 1.7
        s.braidweak = Journey_BraidStack(x, y, self, xoffset = -12, yoffset = self.BRAID_OFFSET)

        # Create talon
        x, y = l.XM + l.XS * 2 + l.XS * decks, h - l.YS - l.YM
        s.talon = WasteTalonStack(x, y, self, max_rounds = 3)
        l.createText(s.talon, "ss")
        s.talon.texts.rounds = MfxCanvasText(self.canvas,
                               self.width / 2, h - l.YM * 2.5,
                               anchor = "center")
        x = x + l.XS * 2
        s.waste = WasteStack(x, y, self)
        l.createText(s.waste, "ss")

        # define stack-groups
        self.sg.talonstacks = [s.talon] + [s.waste]
        self.sg.openstacks = s.foundations + s.rows
        self.sg.dropstacks = [s.braidstrong] + [s.braidweak] + s.rows + [s.waste]


    #
    # game overrides
    #

    def startGame(self):
        self.startDealSample()
        self.base_card = None
        self.updateText()
        for i in range(self.BRAID_CARDS):
            self.s.talon.dealRow(rows = [self.s.braidstrong])
        for i in range(self.BRAID_CARDS):
            self.s.talon.dealRow(rows = [self.s.braidweak])
        self.s.talon.dealRow()
        # deal base_card to foundations, update cap.base_rank
        self.base_card = self.s.talon.getCard()
        to_stack = self.s.foundations[self.base_card.suit * self.gameinfo.decks]
        self.flipMove(self.s.talon)
        self.moveMove(1, self.s.talon, to_stack)
        self.updateText()
        for s in self.s.foundations:
            s.cap.base_rank = self.base_card.rank
        # deal first card to WasteStack
        self.s.talon.dealCards()

    def shallHighlightMatch(self, stack1, card1, stack2, card2):
        return (card1.suit == card2.suit and
                ((card1.rank + 1) % 12 == card2.rank
                or (card2.rank + 1) % 12 == card1.rank))

    def getHighlightPilesStacks(self):
        return ()

    def _restoreGameHook(self, game):
        self.base_card = self.cards[game.loadinfo.base_card_id]
        for s in self.s.foundations:
            s.cap.base_rank = self.base_card.rank

    def _loadGameHook(self, p):
        self.loadinfo.addattr(base_card_id=None)    # register extra load var.
        self.loadinfo.base_card_id = p.load()

    def _saveGameHook(self, p):
        p.dump(self.base_card.id)


    #
    # game extras
    #

    def updateText(self):
        if self.preview > 1 or not self.texts.info:
            return
        if not self.base_card:
            t = ""
        else:
            t = self.RANKS[self.base_card.rank]
            dir = self.getFoundationDir() % 12
            if dir == 1:
                t = t + " Ascending"
            elif dir == 11:
                t = t + " Descending"
        self.texts.info.config(text = t)


# /***********************************************************************
# // Long Journey to Cuddapah
# ************************************************************************/

class LongJourney(Journey):
    Hint_Class = Journey_Hint

    BRAID_CARDS = 20
    BRAID_OFFSET = 0.7



registerGame(GameInfo(15406, Matsya, "Matsya",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15407, Kurma, "Kurma",
                      GI.GT_DASHAVATARA_GANJIFA, 1, -1,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15408, Varaha, "Varaha",
                      GI.GT_DASHAVATARA_GANJIFA, 1, -1,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15409, Narasimha, "Narasimha",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15410, Vamana, "Vamana",
                      GI.GT_DASHAVATARA_GANJIFA, 1, -1,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15411, Parashurama, "Parashurama",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 1,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15412, TenAvatars, "Ten Avatars",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15413, DashavataraCircles, "Dashavatara Circles",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15414, Balarama, "Balarama",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15415, Hayagriva, "Hayagriva",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15416, Shanka, "Shanka",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15417, Journey, "Journey to Cuddapah",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 2,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15418, LongJourney, "Long Journey to Cuddapah",
                      GI.GT_DASHAVATARA_GANJIFA, 2, 2,
                      suits=range(10), ranks=range(12)))

registerGame(GameInfo(15419, Surukh, "Surukh",
                      GI.GT_DASHAVATARA_GANJIFA, 1, 0,
                      suits=range(10), ranks=range(12)))

